@@ -20,8 +20,10 @@
 
 */
 
+require_once('/usr/share/php-getid3/getid3.php');
+
 /* A list of file formats we will recognize */
-$FORMATS = array("mp3", "oga", "ogg", "wav", "flac");
+$FORMATS = array("mp3", "m4a", "oga", "ogg", "wav", "flac");
 $JPLAYER_LIST = "";
 $JPLAYER_OGG = "false";
 
@@ -107,11 +109,30 @@ function split3($path) {
 	return array($artist, $album, $song);
 }
 
+
 function urlencode_album($album) {
 	// Must allow "/" in album names, for nested albums
 	return preg_replace("/%2F/", "/", urlencode($album));
 }
 
+function array_rtrim($a) {
+	// This sort() is annoyingly necessary, as preg_grep() maintains indexes of searched arrays;  this sort will flatten the result
+	sort($a);
+	for ($i=0; $i<sizeof($a); $i++) {
+		$a[$i] = rtrim($a[$i]);
+	}
+	return $a;
+}
+
+function preg_escape($str) {
+	$patterns = array('/\//', '/\^/', '/\./', '/\$/', '/\|/',
+ '/\(/', '/\)/', '/\[/', '/\]/', '/\*/', '/\+/',
+'/\?/', '/\{/', '/\}/', '/\,/');
+	$replace = array('\/', '\^', '\.', '\$', '\|', '\(', '\)',
+'\[', '\]', '\*', '\+', '\?', '\{', '\}', '\,');
+	return preg_replace($patterns, $replace, $str);
+}
+
 $top = sanity_check($_REQUEST["top"]);
 $middle = sanity_check($_REQUEST["middle"]);
 $blank = sanity_check($_REQUEST["blank"]);
@@ -123,7 +144,6 @@ $misc = sanity_check($_REQUEST["misc"]);
 $playlist = sanity_check($_REQUEST["playlist"]);
 $download_album = sanity_check($_REQUEST["download_album"]);
 $search = sanity_check($_REQUEST["search"]);
-$rating = sanity_check($_REQUEST["rating"]);
 $user = sanity_check($_SERVER["PHP_AUTH_USER"]);
 $pw = sanity_check($_SERVER["PHP_AUTH_PW"]);
 
@@ -163,195 +183,130 @@ body {font-size: 13px; font-family: verdana,arial,helvetica,sans-serif; font-wei
 /*****************/
 
 function get_all_artists($search="") {
-	$artists = array();
-	if ($dir = opendir("music")) {
-		while (($artist = readdir($dir)) !== false) {
-			if (is_dir("music/$artist") && visible($artist)) {
-				if (!$search || preg_match("/$search/i", $artist)) {
-					array_push($artists, $artist);
-				}
-			}
+	$artists = array_rtrim(@file("/var/lib/musica/artists"));
+	if ($search) {
+		if ($search == "_random_") {
+			$artist = $artists[rand(0, sizeof($artists) - 1)];
+			$artists = array($artist);
+			print("
+				<script>
+					parent._albums.location.href='?artist=" . urlencode($artist) . "';
+					parent._songs.location.href='?artist=" . urlencode($artist) . "&misc=1';
+				</script>
+			");
+		} else {
+			$str = preg_escape($search);
+			$artists = array_rtrim(preg_grep("/$str/i", $artists));
 		}
 	} else {
-		print("<b>ERROR</b><br>No music found.<br><br>Create a symlink to your music folder at<pre>" . dirname($_SERVER["SCRIPT_FILENAME"]) . "/music</pre>");
-		exit;
+		if (sizeof($artists) == 0) {
+			print("<b>ERROR</b><br>No music found.<br><br>Create a symlink to your music folder at<pre>" . dirname($_SERVER["SCRIPT_FILENAME"]) . "/music</pre>");
+			exit;
+		}
 	}
-	sort($artists);
 	return $artists;
 }
 
 function get_all_albums($search="") {
-	$artists = get_all_artists();
-	$albums = array();
-	for ($i=0; $i<sizeof($artists); $i++) {
-		$a =  get_albums_by_artist($artists[$i]);
-		for ($j=0; $j<sizeof($a); $j++) {
-			list($artist, $album) = split2($a[$j]);
-			if (!$search || preg_match("/$search/i", $album)) {
-				array_push($albums, $a[$j]);
-			}
+	$albums = array_rtrim(@file("/var/lib/musica/albums"));
+	if ($search) {
+		if ($search == "_random_") {
+			$album = $albums[rand(0, sizeof($albums) - 1)];
+			$albums = array($album);
+			list($artist, $album) = split2($album);
+			print("
+				<script>
+					parent._artists.location.href='?search=" . urlencode($artist) . "&artist=_all';
+					parent._songs.location.href='?artist=" . urlencode($artist) . "&album=" . urlencode($album) . "';
+				</script>
+			");
+		} else {
+			$str = preg_escape($search);
+			$albums = array_rtrim(preg_grep("/.*\/.*$str/i", $albums));
 		}
 	}
-	sort($albums);
 	return $albums;
 }
 
-function get_all_songs($search="", $rating=0) {
-	if ($rating) {
-		$ratings = get_all_ratings();
-		$artists = array_keys($ratings);
-	} else {
-		$artists = get_all_artists();
-	}
-	$songs = array();
-	$images = array();
-	for ($i=0; $i<sizeof($artists); $i++) {
-		list($s, $images) = get_songs_by_album($artists[$i]);
-		for ($j=0; $j<sizeof($s); $j++) {
-			list($artist, $album, $song) = split3($s[$j]);
-			if (!$search || preg_match("/$search/", $song)) {
-				if (!$rating || $ratings["$artist"] >= $rating) {
-					array_push($songs, $s[$j]);
-				}
-			}
+function get_all_songs($search="") {
+	$songs = @file("/var/lib/musica/songs");
+	if ($search) {
+		if ($search == "_random_") {
+			$song = $songs[rand(0, sizeof($songs) - 1)];
+			$songs = array($song);
+			list($artist, $album, $song) = split3($song);
+			print("
+				<script>
+					parent._artists.location.href='?search=" . urlencode($artist) . "&artist=_all';
+					parent._albums.location.href='?search=" . urlencode($album) . "&artist=_search&album=_search';
+				</script>
+			");
+			print_song_header($artist, $album, $search);
+		} else {
+			$str = preg_escape($search);
+			$songs = array_rtrim(preg_grep("/.*\/.*\/.*$str/i", $songs));
 		}
 	}
-	$albums = get_all_albums();
-	for ($i=0; $i<sizeof($albums); $i++) {
-		list($artist, $album) = split2($albums[$i]);
-		list($s, $images) = get_songs_by_album($artist, $album);
-		for ($j=0; $j<sizeof($s); $j++) {
-			list($artist, $album, $song) = split3($s[$j]);
-			if (!$search || preg_match("/$search/i", $song)) {
-				if (!$rating || $ratings["$artist"] >= $rating) {
-					array_push($songs, $s[$j]);
-				}
-			}
-		}
-	}
-	sort($songs);
+	$songs = array_rtrim($songs);
 	return $songs;
 }
 
 function get_albums_by_artist($artist) {
-	$albums = array();
-	if (is_dir("music/$artist")) {
-		$albums = findsubdirs("music/$artist");
-	}
-	array_shift($albums);
-	for ($i=0; $i<sizeof($albums); $i++) {
-		if (! has_files($albums[$i])) {
-			$albums[$i] = "";
-		} else {
-			$albums[$i] = preg_replace("/^music\//", "", $albums[$i]);
-			$albums[$i] = preg_replace("/\/$/", "", $albums[$i]);
-		}
-	}
-	sort($albums);
+	$str = preg_escape($artist);
+	$albums = array_rtrim(preg_grep("/^$str\//", @file("/var/lib/musica/albums")));
 	return $albums;
 }
 
-function get_all_ratings() {
-	$file = "music/.ratings";
-	$ratings = array();
-	if (file_exists($file)) {
-		$lines = @file($file);
-	} else {
-		return $ratings;
-	}
-	foreach ($lines as $line) {
-		list($a, $r) = preg_split("/\t/", $line);
-		$r = preg_replace("/\s+/", "", $r);
-		$ratings["$a"] = $r;
-	}
-	return $ratings;
-}
-
-function get_rating($artist) {
-	$ratings = get_all_ratings();
-	$r = $ratings["$artist"];
-	if (!isset($r)) {
-		$r = 0;
-	}
-	$str = "";
-	for ($i=1; $i<=$r; $i++) {
-		$str .= "<a href=?rating=$i&artist=" . urlencode($artist) . "><img border=0 src=silk/star.png></a> ";
-	}
-	return $str;
-}
-
-function set_rating($artist, $rating) {
-	if ($rating>=0 && $rating<=5) {
-		$ratings = get_all_ratings();
-		$ratings["$artist"] = $rating;
-		$artists = array_keys($ratings);
-		sort($artists);
-		$fp = @fopen("music/.ratings", "w");
-		if (!$fp) {
-			print("<b>ERROR</b><br>Incorrect permissions on .ratings<p>To use this feature, the file<br><pre>  " . dirname($_SERVER["SCRIPT_FILENAME"]) . "/music/.ratings</pre>must be readable and writeable by the user<pre>  ". $_ENV["APACHE_RUN_USER"] . "</pre></p>");
-			exit;
-		}
-		foreach ($artists as $a) {
-			fprintf($fp, "%s\t%s\n", $a, $ratings["$a"]);
-		}
-		fclose($fp);
-	}
-}
-
 function get_songs_by_album($artist, $album="") {
-	$songs = array();
-	$images = array();
-	if (is_dir("music/$artist/$album")) {
-		if ($dir = opendir("music/$artist/$album")) {
-			while (($file = readdir($dir)) !== false){
-				if (visible($artist) && visible($album) && visible($file)) {
-					if (is_song("music/$artist/$album/$file")) {
-						array_push($songs, "$artist/$album/$file");
-					} elseif (exif_imagetype("music/$artist/$album/$file")) {
-						array_push($images, "music/$artist/$album/$file");
-					}
-				}
-			}
-			closedir($dir);
-		}
+	$str = "$artist/$album";
+	if ($album) {
+		$str .= "/";
 	}
-	sort($songs);
-	return array($songs, $images);
+	$str = preg_escape($str);
+	$songs = array_rtrim(preg_grep("/^$str/", @file("/var/lib/musica/songs")));
+/*
+	if (sizeof(preg_grep("/^[0-9]/", $songs)) < 1) {
+		$songs = sort_by_id3_tags($songs);
+	}
+*/
+	return $songs;
 }
 
-function get_songs_by_artist($artist) {
-	$songs = array();
-	$images = array();
-	list($s, $images) = get_songs_by_album($artist);
-	for ($i=0; $i<sizeof($s); $i++) {
-		list($artist, $album, $song) = split3($s[$i]);
-		if (!$search || preg_match("/$search/", $song)) {
-			if (!$rating || $ratings["$artist"] >= $rating) {
-				array_push($songs, $s[$i]);
+function get_images_by_album($artist, $album="") {
+	$str = preg_escape("$artist/$album/");
+	$images = array_rtrim(preg_grep("/^$str/", @file("/var/lib/musica/images")));
+	if (sizeof($images) < 1) {
+		$songs = scandir("music/$artist/$album/");
+		for ($i=0; $i<sizeof($songs); $i++) {
+			if ($songs[$i] == "." || $songs[$i] == "..") { continue; }
+			$getID3 = new getID3;
+			$getID3->analyze("music/$artist/$album/$songs[$i]");
+			if (isset($getID3->info['id3v2']['APIC'][0]['data'])) {
+				$cover = $getID3->info['id3v2']['APIC'][0]['data'];
+			} elseif (isset($getID3->info['id3v2']['PIC'][0]['data'])) {
+				$cover = $getID3->info['id3v2']['PIC'][0]['data'];
+			} else {
+				$cover = null;
+			}
+			if ($cover) {
+				array_push($images, "<center><img width=200 src=data:image/gif;base64," . base64_encode($cover) . "></center><br>");
 			}
 		}
 	}
-	$albums = get_albums_by_artist($artist);
-	for ($i=0; $i<sizeof($albums); $i++) {
-		list($artist, $album) = split2($albums[$i]);
-		list($s, $images) = get_songs_by_album($artist, $album);
-		for ($j=0; $j<sizeof($s); $j++) {
-			list($artist, $album, $song) = split3($s[$j]);
-			if (!$search || preg_match("/$search/i", $song)) {
-				if (!$rating || $ratings["$artist"] >= $rating) {
-					array_push($songs, $s[$j]);
-				}
-			}
-		}
-	}
-	sort($songs);
+	return $images;
+}
+
+function get_misc_songs_by_artist($artist) {
+	$str = "$artist";
+	$songs = array_rtrim(preg_grep("/^$str\//", @file("/var/lib/musica/songs")));
+	// Prune those that are part of albums
+	$songs = array_rtrim(preg_grep("/\/.*\//", $songs, PREG_GREP_INVERT));
 	return $songs;
 }
 
 function get_size_of_album($artist, $album) {
 	$songs = array();
-	$images = array();
-	list($songs, $images) = get_songs_by_album($artist, $album);
+	$songs = get_songs_by_album($artist, $album);
 	$size = 0;
 	for ($i=0; $i<sizeof($songs); $i++) {
 		$size += filesize("music/$songs[$i]");
@@ -361,13 +316,6 @@ function get_size_of_album($artist, $album) {
 	return "~$size MB";
 }
 
-function get_random_artist() {
-	$artists = get_all_artists();
-	$artist = $artists[rand(0, sizeof($artists) - 1)];
-	$artist = preg_replace("/\\\'/", "'", $artist);
-	return $artist;
-}
-
 function get_counts() {
 	# Updated by an hourly cronjob
 	$artists = @file("/var/lib/musica/artists.count");
@@ -377,16 +325,6 @@ function get_counts() {
 	return $counts;
 }
 
-function get_artist_ratings() {
-	$ratings = array();
-	list($r) = @file("music/.ratings");
-	for ($i=0; $i<sizeof($r); $i++) {
-		list($artist, $rating) = preg_split("/\t/", $r, 2);
-		$ratings[$artist] = $rating;
-	}
-	return $ratings;
-}
-
 /*******************/
 /* print functions */
 /*******************/
@@ -398,13 +336,18 @@ function print_artist($artist) {
 	}
 }
 
-function print_album($artist, $album) {
+function print_album($artist, $album, $with_artist=0) {
 	if (visible($artist) && visible($album) && isset($album)) {
 		$href = "?artist=" . urlencode($artist) . "&album=" . urlencode_album($album);
-		print("<li><a href=$href target=_songs><img border=0 src=silk/cd.png>&nbsp;$album</a><br>");
+		if ($with_artist) {
+			print("<li><a href=\"$href\" target=_songs><img src=silk/group.png>&nbsp;$artist</a><br>&nbsp;&nbsp;<a href=$href target=_songs><img border=0 src=silk/cd.png>&nbsp;$album</a><br>");
+		} else {
+			print("<li><a href=$href target=_songs><img border=0 src=silk/cd.png>&nbsp;$album</a><br>");
+		}
 	}
 }
 
+
 function print_song($artist, $album, $song) {
 	global $PREAMBLE, $JPLAYER_LIST, $JPLAYER_OGG;
 	print("<noscript>");
@@ -427,38 +370,33 @@ function print_artists($search="") {
 	print("<ol>");
 	$artists = get_all_artists($search);
 	for ($i=0; $i<sizeof($artists); $i++) {
-		print_artist($artists[$i]);
+		print_artist(rtrim($artists[$i]));
 	}
-	print("
-		</ol>
-		<hr>
-		<ul>
-		  <li><a href=?artist=_random target=_albums>Random Artist</a></li>
-		  <li>Playlist of all songs with artist rating</li>
-		  <ul>
-	");
-	for ($i=5; $i>=1; $i--) {
-		print("<li><a href=?playlist=$i target=_songs>");
-		for ($j=1; $j<=$i; $j++) {
-			print("<img border=0 src=silk/star.png>");
-		}
-		print("</a></li>");
-	}
-	print("</ul></ul>");
+}
+
+function print_song_header($artist, $album, $search="") {
+	global $_SERVER;
+	print("<center><table><tr><td><p align=left><img src=silk/group.png>&nbsp;<b><big>$artist</b></big><br><img src=silk/cd.png>&nbsp;<b>$album</b><br>\n");
+	$wiki = preg_replace("/\s*\(.*/", "", $album);
+	$wiki = urlencode($wiki);
+	$wiki = preg_replace("/\+/", "_", $wiki);
+	$size = get_size_of_album($artist, $album);
+	print("<center>
+		<a title='Wikipedia' target=_new href=http://en.wikipedia.org/wiki/$wiki><img src=silk/book_open.png border=0></a>
+		<a title='Launch player in a new window' target=_new href=" . $_SERVER["REQUEST_URI"] . "&search=$search&popout=1><img src=silk/application_double.png border=0></a>
+		<a title='Save playlist' href=?playlist=1&artist=" . urlencode($artist) . "&album=" . urlencode_album($album) . "><img src=silk/control_play_blue.png border=0></a>
+		<a title='Download entire album ($size)' href=?download_album=1&artist=" . urlencode($artist) . "&album=" . urlencode_album($album) . "><img src=silk/disk.png border=0></a>
+	       </center>
+	</p></td></tr></table></center>");
 }
 
 function print_albums_by_artist($artist) {
-	global $rating;
-	if (is_numeric($rating)) {
-		set_rating($artist, $rating);
-	}
 	$albums = get_albums_by_artist($artist);
 	print("<p align=center><img src=silk/group.png>&nbsp;<big><b>$artist</b></big><br>");
 	$wiki = preg_replace("/\s*\(.*/", "", $artist);
 	$wiki = urlencode($wiki);
 	$wiki = preg_replace("/\+/", "_", $wiki);
-	print("<a target=_new href=http://en.wikipedia.org/wiki/$wiki><img width=16 heigh=16 src=silk/book_open.png border=0> Wikipedia</a><br>");
-	print( get_rating($artist) . "</p><ol>");
+	print("<a target=_new href=http://en.wikipedia.org/wiki/$wiki><img width=16 heigh=16 src=silk/book_open.png border=0> Wikipedia</a><br></p><ol>");
 	$loadmisc = 0;
 	for ($i=0; $i<sizeof($albums); $i++) {
 		list($artist, $album) = preg_split("/\//", $albums[$i], 2);
@@ -472,25 +410,20 @@ function print_albums_by_artist($artist) {
 
 function print_albums_by_search($search) {
 	$albums = get_all_albums($search);
-	print("<center><big><b>Matching Artists or Albums</b></big></center><ol>");
 	print("<ol>");
 	for ($i=0; $i<sizeof($albums); $i++) {
-		list($artist, $album) = split2($albums[$i]);
-		print_artist($artist);
-		print_album($artist, $album);
+		list($artist, $album) = split2(rtrim($albums[$i]));
+		print_album($artist, $album, 1);
 	}
 	print("</ol>");
 }
 
 function print_songs_by_album($artist, $album) {
 	global $JPLAYER_LIST, $JPLAYER_OGG;
-	print("<p align=center><img src=silk/group.png>&nbsp;<b><big>$artist</b></big><br><img src=silk/cd.png>&nbsp;<b>$album</b><br>\n");
-	$wiki = preg_replace("/\s*\(.*/", "", $album);
-	$wiki = urlencode($wiki);
-	$wiki = preg_replace("/\+/", "_", $wiki);
-	print("<a target=_new href=http://en.wikipedia.org/wiki/$wiki><img width=16 heigh=16 src=silk/book_open.png border=0> Wikipedia</a><br></p>");
-	list($songs, $images) = get_songs_by_album($artist, $album);
-	print_album_cover($images, "$artist/$album");
+	print_song_header($artist, $album);
+	$songs = get_songs_by_album($artist, $album);
+	$images = get_images_by_album($artist, $album);
+	print_album_cover($images);
 	for ($i=0; $i<sizeof($songs); $i++) {
 		list($artist, $album, $song) = split3($songs[$i]);
 		print_song($artist, $album, $song);
@@ -498,19 +431,12 @@ function print_songs_by_album($artist, $album) {
 	if ($JPLAYER_LIST != "") {
 		include("jplayer.php");
 	}
-	$size = get_size_of_album($artist, $album);
-        print("<p align=right>
-<a target=_new href=" . $_SERVER["REQUEST_URI"] . "&popout=1><img src=silk/application_double.png border=0>&nbsp;play in a new window</a><br>
-<a href=?playlist=1&artist=" . urlencode($artist) . "&album=" . urlencode_album($album) . "><img src=silk/control_play_blue.png border=0>&nbsp;play album</a><br>
-<a href=?download_album=1&artist=" . urlencode($artist) . "&album=" . urlencode_album($album) . "><img src=silk/disk.png border=0>&nbsp;download</a> $size
-	");
 }
 
 function print_misc_songs_by_artist($artist) {
 	global $JPLAYER_LIST, $JPLAYER_OGG, $_SERVER;
 	print("<center><img src=silk/group.png>&nbsp;<big><b>$artist</b></big><br><img src=silk/music.png>&nbsp;<b>Miscellaneous</b></center><br>");
-	list($songs, $images) = get_songs_by_album($artist);
-	print_album_cover($images, $artist);
+	$songs = get_misc_songs_by_artist($artist);
 	for ($i=0; $i<sizeof($songs); $i++) {
 		list($artist, $album, $song) = split3($songs[$i]);
 		print_song($artist, $album, $song);
@@ -518,9 +444,6 @@ function print_misc_songs_by_artist($artist) {
 	if ($JPLAYER_LIST != "") {
 		include("jplayer.php");
 	}
-	print("<p align=right>
-<a target=_new href=" . $_SERVER["REQUEST_URI"] . "&popout=1><img src=silk/application_double.png border=0>&nbsp;play in a new window</a><br>
-<a href=?playlist=1&artist=" . urlencode($artist) . "><img src=silk/control_play_blue.png border=0>&nbsp;play all by artist</a></p> ");
 }
 
 function print_songs_by_search($search) {
@@ -528,7 +451,7 @@ function print_songs_by_search($search) {
         print("<ol>");
 	$songs = get_all_songs($search);
 	for ($i=0; $i<sizeof($songs); $i++) {
-		list($artist, $album, $song) = split3($songs[$i]);
+		list($artist, $album, $song) = split3(rtrim($songs[$i]));
 		//print_artist($artist);
 		//print_album($artist, $album);
 		print_song($artist, $album, $song);
@@ -540,44 +463,23 @@ function print_songs_by_search($search) {
 	print("</ol>");
 }
 
-function print_album_cover($images, $dir) {
+function print_album_cover($images) {
 	$width = 200;
-	$js = "src = [";
-	for ($i=0; $i<sizeof($images); $i++) {
-		if ($i > 0) {
-			$js .= ",";
-		}
-		$js .= "'" . $images[$i] . "'";
-	}
-	$js .= "]";
-	if ($i>= 1) {
-		# From http://www.javascriptkit.com/script/script2/incrementslide.shtml
-		print("
-<script>
-" . $js . "
-pics=[]; i=0;
-function switch_pic() {
-  var n=(i+1)%src.length;
-  if (pics[n] && (pics[n].complete || pics[n].complete==null)) {
-    document['albumart'].src = pics[i=n].src;
-  }
-  pics[n=(i+1)%src.length] = new Image;
-  pics[n].src = src[n];
-  setTimeout('switch_pic()',4000);
-} onload = function(){
-  if (document.images)
-    switch_pic();
-}
-</script>");
-		$img = "<center><img name=albumart width=$width src='$images[0]'></center><br>";
-	} else {
-		$tmp = "/var/lib/musica/tmp";
-		$output = `eyeD3 -i $tmp "/usr/share/musica/music/$dir" 2>&1`;
-		$output = preg_replace("/.*Writing /", "", $output);
-		list($filename, $rest) = preg_split("/\.\.\./", $output);
-		if (file_exists($filename)) {
-			$img = "<center><img width=$width src=data:image/gif;base64," . base64_encode(file_get_contents("$filename")) . $img . "></center><br>";
-			unlink($filename);
+	if (sizeof($images) >= 1) {
+		$max_size = 0;
+		# Uses the largest (presumably the best?) image
+		for ($i=0; $i<sizeof($images); $i++) {
+			if (preg_match("/^<center>/", $images[$i])) {
+				$this_size = strlen($images[$i]);
+				$this_img = $images[$i];
+			} else {
+				$this_size = filesize("music/$images[$i]");
+				$this_img = "<center><img name=albumart width=$width src='music/$images[$i]'></center><br>";
+			}
+			if ($this_size > $max_size) {
+				$max_size = $this_size;
+				$img = $this_img;
+			}
 		}
 	}
 	print($img);
@@ -664,19 +566,19 @@ if ($top) {
       <form method=post action=?artist=_all target=_artists>
         <input type=text name=search>
         <input type=submit value=find>
-        <input type=submit value=all onfocus=javascript:document.forms[0].search.value=\"\"></form>
+        <input type=submit value=random onfocus=javascript:document.forms[0].search.value=\"_random_\"></form>
     </td>
     <td>
       <form method=post action=?artist=_search&album=_search target=_albums>
         <input type=text name=search>
         <input type=submit value=find>
-        <input type=submit value=all onfocus=javascript:document.forms[0].search.value=\"\"></form>
+        <input type=submit value=random onfocus=javascript:document.forms[1].search.value=\"_random_\"></form>
     </td>
     <td>
       <form method=post action=?artist=_search&album=_search&song=_search target=_songs>
         <input type=text name=search>
         <input type=submit value=find>
-        <input type=submit value=all onfocus=javascript:document.forms[0].search.value=\"\"></form>
+        <input type=submit value=random onfocus=javascript:document.forms[2].search.value=\"_random_\"></form>
     </td>
   </tr>
 </table></body>");
@@ -714,9 +616,6 @@ if ($artist == "_all") {
 /*************************************************************************/
 /* Album Frame: Display list of albums */
 if ($artist && !$album && !$misc && !$playlist && !$download_album) {
-	if ($artist == "_random") {
-		$artist = get_random_artist();
-	}
 	print_albums_by_artist($artist);
 	exit;
 }
@@ -759,7 +658,7 @@ if ($playlist && !$artist && !$album) {
 	header("Content-type: application/download");
 	header("Content-disposition: attachment; filename=$filename");
 	for ($i=0; $i<sizeof($songs); $i++) {
-		$line = $PREAMBLE . urlencode($songs[$i]);
+		$line = $PREAMBLE . urlencode(rtrim($songs[$i]));
 		$line = preg_replace("/\+/", "%20", $line);
 		$line = preg_replace("/\%2F/", "/", $line);
 		print("$line\n");
@@ -779,10 +678,10 @@ if ($playlist) {
 		array_push($songs, "$artist/$album/$song");
 		$filename = urlencode($artist) . "_" . urlencode($album) . "_" . urlencode($song) . ".m3u";
 	} elseif ($album) {
-		list($songs, $images) = get_songs_by_album($artist, $album);
+		$songs = get_songs_by_album($artist, $album);
 		$filename = urlencode($artist) . "_" . urlencode($album) . ".m3u";
 	} elseif ($artist) {
-		$songs = get_songs_by_artist($artist);
+		$songs = get_misc_songs_by_artist($artist);
 		$filename = urlencode($artist) . ".m3u";
 	}
 	header("Content-type: application/download");
